try:
    from . import _logical_writers
except ImportError:
    from . import _logical_writers_py as _logical_writers  # type: ignore

LOGICAL_WRITERS = _logical_writers.LOGICAL_WRITERS

__all__ = [
    'LOGICAL_WRITERS'
]



PRIMITIVES = {
    'boolean',
    'bytes',
    'double',
    'float',
    'int',
    'long',
    'null',
    'string',
}

RESERVED_PROPERTIES = {
    'type',
    'name',
    'namespace',
    'fields',  # Record
    'items',  # Array
    'size',  # Fixed
    'symbols',  # Enum
    'values',  # Map
    'doc',
}

OPTIONAL_FIELD_PROPERTIES = {
    "doc",
    "aliases",
    "default",
}

RESERVED_FIELD_PROPERTIES = {
    "type",
    "name"
} | OPTIONAL_FIELD_PROPERTIES


class UnknownType(ValueError):
    def __init__(self, name):
        super(UnknownType, self).__init__(name)
        self.name = name


class SchemaParseException(Exception):
    pass



from collections import namedtuple
import json


class ValidationErrorData(namedtuple('ValidationErrorData',
                                     ['datum', 'schema', 'field'])):
    def __str__(self):
        # update field for prettier printing
        field = self.field
        if field is None:
            field = ''

        if self.datum is None:
            return 'Field({field}) is None' \
                   ' expected {schema}'.format(field=field,
                                               schema=self.schema)

        return u'{field} is <{datum}> of type ' \
               u'{given_type} expected {schema}'. \
            format(datum=self.datum, given_type=type(self.datum),
                   schema=self.schema, field=field)


class ValidationError(Exception):
    def __init__(self, *errors):
        message = json.dumps([str(e) for e in errors],
                             indent=2,
                             ensure_ascii=False)
        super(ValidationError, self).__init__(message)
        self.errors = errors
